#ifndef BASICS_H
#define BASICS_H

#include <stdlib.h>
#include <nrf52.h>
#include <chip.h>
#include "util.h"

typedef enum {
	PNUL = -1,
	PORT0 = 0
} Port_t;

typedef enum {
	NOT_A_PIN = -1, // not under control of a peripheral
	PIN_ANALOG, // pin is controlled by the associated signal of peripheral B
	PIN_DIGITAL, // pin is controlled by PORT
	PIN_INPUT, // pin is controlled by PORT and is an INPUT
	PIN_INPUT_PULLUP, // pin is controlled by PORT and is an INPUT with internal PULL-UP
	PIN_INPUT_PULLDOWN, // pin is controlled by PORT and is an INPUT with internal PULL-DOWN
	PIN_OUTPUT // pin is controlled by PORT and is an OUTPUT
} PinType_t;

typedef enum {
	NO_ADC = -1,
	ADC_A0 = 1,
	ADC_A1,
	ADC_A2,
	ADC_A3,
	ADC_A4,
	ADC_A5,
	ADC_A6,
	ADC_A7
} AnalogChannel_t;

typedef enum {
	NOT_ON_PWM = -1,
	PWM0 = 0x00,
	PWM1 = 0x01,
	PWM2 = 0x02,
	PWM3 = 0x03,
	PWM4 = 0x10,
	PWM5 = 0x11,
	PWM6 = 0x12,
	PWM7 = 0x13,
	PWM8 = 0x20,
	PWM9 = 0x21,
	PWM10 = 0x22,
	PWM11 = 0x23
} PWMChannel_t;

// pin attributes
#define PIN_ATTR_NONE (0UL << 0)
#define PIN_ATTR_ANALOG (1UL << 1)
#define PIN_ATTR_DIGITAL (1UL << 2)
#define PIN_ATTR_PWM (1UL << 3)

// pin statuses
#define INPUT (0UL)
#define OUTPUT (1UL)
#define INPUT_PULLUP (2UL)
#define INPUT_PULLDOWN (4UL)

typedef struct {
	Port_t port;
	uint8_t pin;
	PinType_t pin_type;
	uint8_t pin_attribute;
	AnalogChannel_t analog_channel;
	PWMChannel_t pwm_channel;
} PinDescription_t;

#endif
