OUTPUT_FORMAT("elf32-littlearm", "elf32-bigarm", "elf32-littlearm")
SEARCH_DIR(.)
GROUP(-lgcc -lc -lnosys)
ENTRY(Reset_Handler)

/* Memory Space Definitions */
MEMORY {
	FLASH (rx)	: ORIGIN = 0x00000000, LENGTH = 0x00080000
	RAM (rwx)	: ORIGIN = 0x20000000, LENGTH = 0x00010000
}

/* The stack size used by the application. Maximum stack used by S132's spec is 1536 bytes. 2KB
 * should be enough.
 */
__stack_size = 0x800;

/* Set stack top to end of RAM, and move stack limit down by size of stack_dummy section. */
PROVIDE(__stack_top = ORIGIN(RAM) + LENGTH(RAM));
PROVIDE(__stack_limit = __stack_top - __stack_size);
PROVIDE(__stack = __stack_top);

/* Section Definitions */
SECTIONS {
	.text : {
		__text_start = .;
		KEEP(*(.isr_vector))
		*(.text*)
		KEEP(*(.init))
		KEEP(*(.fini))
		
		/* .ctors */
		*crtbegin.o(.ctors)
		*crtbegin?.o(.ctors)
		*(EXCLUDE_FILE(*crtend?.o *crtend.o) .ctors)
		*(SORT(.ctors.*))
		*(.ctors)
		
		/* .dtors */
		*crtbegin.o(.dtors)
		*crtbegin?.o(.dtors)
		*(EXCLUDE_FILE(*crtend?.o *crtend.o) .dtors)
		*(SORT(.dtors.*))
		*(.dtors)
		*(.rodata*)
		KEEP(*(.eh_frame*))
	} > FLASH
	
	.ARM.extab : {
		*(.ARM.extab* .gnu.linkonce.armextab.*)
	} > FLASH
	
	__exidx_start = .;
	
	.ARM.exidx : {
		*(.ARM.exidx* .gnu.linkonce.armexidx.*)
	} > FLASH
	
	__exidx_end = .;
	__etext = .;
	
	.data : AT (__etext) {
		__data_start = .;
		*(vtable)
		*(.data*)
		. = ALIGN(4);
		
		/* Pre Init Data */
		PROVIDE_HIDDEN(__preinit_array_start = .);
		KEEP(*(.preinit_array))
		PROVIDE_HIDDEN(__preinit_array_end = .);
		. = ALIGN(4);
		
		/* Init Data */
		PROVIDE_HIDDEN(__init_array_start = .);
		KEEP(*(SORT(.init_array.*)))
		KEEP(*(.init_array))
		PROVIDE_HIDDEN(__init_array_end = .);
		. = ALIGN(4);
		
		/* Finit Data */
		PROVIDE_HIDDEN(__fini_array_start = .);
		KEEP(*(SORT(.fini_array.*)))
		KEEP(*(.fini_array))
		PROVIDE_HIDDEN(__fini_array_end = .);
		KEEP(*(.jcr*))
		. = ALIGN(4);
		
		/* All Data End */
		__data_end = .;
	} > RAM
	
	.bss : {
		. = ALIGN(4);
		__bss_start = .;
		*(.bss*)
		*(COMMON)
		. = ALIGN(4);
		__bss_end = .;
	} > RAM
	
	.heap (COPY) : {
		__end = .;
		PROVIDE(end = .);
		*(.heap*)
		__heap_limit = .;
	} > RAM
	
	/* .stack dummy section doesn't contain any symbols. It is only used for the linker to
	 * calculate the size of the .stack section, and assign values to the stack symbols
	 * later.
	 */
	 .stack_dummy (COPY) : {
	 	*(.stack*)
	 } > RAM
	 
	 /* Set stack stop to end of RAM, and stack limit move down by size of stack_dummy section. */
	 __stack_top = ORIGIN(RAM) + LENGTH(RAM);
	 __stack_limit = __stack_top - SIZEOF(.stack_dummy);
	 PROVIDE(__stack = __stack_top);
	 
	 /* Check if data + heap + stack exceeds RAM limit. */
	 ASSERT(__stack_limit >= __heap_limit, "RAM region overflowed with stack.")
}