#ifndef COMPONENT_CLOCK_H
#define COMPONENT_CLOCK_H

/** SOFTWARE API DEFINITION FOR Clock **/

typedef struct {
	__O uint32_t TASKS_HFCLKSTART; // Offset: 0x000 - Start HFCLK Crystal Oscillator
	__O uint32_t TASKS_HFCLKSTOP; // Offset: 0x004 - Stop HFCLK Crystal Oscillator
	__O uint32_t TASKS_LFCLKSTART; // Offset: 0x008 - Start LFCLK Source
	__O uint32_t TASKS_LFCLKSTOP; // Offset: 0x00C - Stop LFCLK Source
	__O uint32_t TASKS_CAL; // Offset: 0x010 - Start Calibration of LFRC Oscillator
	__O uint32_t TASKS_CTSTART; // Offset: 0x014 - Start Calibration Timer
	__O uint32_t TASKS_CTSTOP; // Offset: 0x018 - Stop Calibration Timer
	__I uint32_t RESERVED0[57];
	__IO uint32_t EVENTS_HFCLKSTARTED; // Offset: 0x0F8 - HFCLK Oscillator Started
	__IO uint32_t EVENTS_LFCLKSTARTED; // Offset: 0x0FC - LFCLK Started
	__I uint32_t RESERVED1;
	__IO uint32_t EVENTS_DONE; // Offset: 0x104 - Calibration of LFCLK RC Oscillator Complete Event
	__IO uint32_t EVENTS_CTTO; // Offset: 0x108 - Calibration Timer Timeout
	__I uint32_t RESERVED2[124];
	__IO uint32_t INTENSET; // Offset: 0x2FC - Interrupt Enable Set Register
	__IO uint32_t INTENCLR; // Offset: 0x300 - Interrupt Enable Clear Register
	__I uint32_t RESERVED3[63];
	__I uint32_t HFCLKRUN; // Offset: 0x400 - HFCLK Started Register
	__I uint32_t HFCLKSTAT; // Offset: 0x404 - HFCLK Status Register
	__I uint32_t RESERVED4;
	__I uint32_t LFCLKRUN; // Offset: 0x40C - LFCLK Started Register
	__I uint32_t LFCLKSTAT; // Offset: 0x410 - LFCLK Status Register
	__I uint32_t LFCLKSRCCOPY; // Offset: 0x414 - Copy of LFCLKSRC Register
	__I uint32_t RESERVED5[62];
	__IO uint32_t LFCLKSRC; // Offset: 0x510 - Low Frequency Clock Source Register
	__I uint32_t RESERVED6[7];
	__IO uint32_t CTIV; // Offset: 0x530 - Calibration Timer Interval Register
	__I uint32_t RESERVED7[8];
	__IO uint32_t TRACECONFIG; // Offset: 0x554 - Trace Port Debug Interface Clocking Options
} Clock_t;

/* LFCLKSRC - Low Frequency Clock Source Register */
#define CLOCK_LFCLKSRC_SRC_Pos 0
#define CLOCK_LFCLKSRC_SRC_Msk (0x3UL << CLOCK_LFCLKSRC_SRC_Pos)
#define CLOCK_LFCLKSRC_SRC_XTAL (1UL) // 32.768kHz Crystal Oscillator

#endif
