#ifndef COMPONENT_POWER_H
#define COMPONENT_POWER_H

/** SOFTWARE API DEFINITION FOR Power Controller **/

typedef struct {
	__IO uint32_t POWERCTRL; // Offset: 0x00 - RAM0 Power Control Register
	__O uint32_t POWERSET; // Offset: 0x04 - RAM0 Power Control Set Register
	__O uint32_t POWERCLR; // Offset: 0x08 - RAM0 Power Control Clear Register
	__I uint32_t RESERVED0;
} Power_RAM_t;

typedef struct {
	__I uint32_t RESERVED0[30];
	__O uint32_t TASKS_CONSTLAT; // Offset: 0x078 - Enable Constant Latency Mode
	__O uint32_t TASKS_LOWPWR; // Offset: 0x07C - Enable Low Power Mode (Variable Latency)
	__I uint32_t RESERVED1[34];
	__IO uint32_t EVENTS_POFWARN; // Offset: 0x108 - Power Failure Warning
	__I uint32_t RESERVED2[2];
	__IO uint32_t EVENTS_SLEEPENTER; // Offset: 0x114 - CPU Entered WFI/WFE Sleep
	__IO uint32_t EVENTS_SLEEPEXIT; // Offset: 0x118 - CPU Exited WFI/WFE Sleep
	__I uint32_t RESERVED3[122];
	__IO uint32_t INTENSET; // Offset: 0x304 - Interrupt Enable Set Register
	__IO uint32_t INTENCLR; // Offset: 0x308 - Interrupt Enable Clear Register
	__I uint32_t RESERVED4[61];
	__IO uint32_t RESETREAS; // Offset: 0x400 - Reset Reason Register
	__I uint32_t RESERVED5[9];
	__I uint32_t RAMSTATUS; // Offset: 0x424 - RAM Status Register
	__I uint32_t RESERVED6[53];
	__O uint32_t SYSTEMOFF; // Offset: 0x4FC - System OFF Register
	__I uint32_t RESERVED7[3];
	__IO uint32_t POFCON; // Offset: 0x50C - Power Failure Comparator Configuration
	__I uint32_t RESERVED8[2];
	__IO uint32_t GPREGRET; // Offset: 0x518 - General Purpose Retention Register
	__IO uint32_t GPREGRET2; // Offset: 0x51C - General Purpose Retention Register 2
	__IO uint32_t RAMON; // Offset: 0x520 - RAM On/Off Register
	__I uint32_t RESERVED9[11];
	__IO uint32_t RAMONB; // Offset: 0x550 - RAM On/Off Register B
	__I uint32_t RESERVED10[8];
	__IO uint32_t DCDCEN; // Offset: 0x574 - DC/DC Enable Register
	__I uint32_t RESERVED11[225];
	Power_RAM_t RAM[8]; // Offset: 0x8FC - RAM0 Registers
} Power_t;

/* RESETREAS - Reset Reason Register */
#define POWER_RESETREAS_RESETPIN_Pos (0UL)
#define POWER_RESETREAS_RESETPIN_Msk (0x01U << POWER_RESETREAS_RESETPIN_Pos)

#endif
