#ifndef COMPONENT_RTC_H
#define COMPONENT_RTC_H

/** SOFTWARE API DEFINITION FOR Real-Time Counter (RTC) **/

typedef struct {
	__O uint32_t TASKS_START; // Offset: 0x000 - Start RTC Counter
	__O uint32_t TASKS_STOP; // Offset: 0x004 - Stop RTC Counter
	__O uint32_t TASKS_CLEAR; // Offset: 0x008 - Clear RTC Counter
	__O uint32_t TASKS_TRIGOVRFLW; // Offset: 0x00C - Set COUNTER to 0xFFFFF0
	__I uint32_t RESERVED0[60];
	__IO uint32_t EVENTS_TICK; // Offset: 0x100 - Event on COUNTER Increment
	__IO uint32_t EVENTS_OVRFLW; // Offset: 0x104 - Event on COUNTER Overflow
	__I uint32_t RESERVED1[14];
	__IO uint32_t EVENTS_COMPARE[4]; // Offset: 0x140 - Compare Event on CC Match
	__I uint32_t RESERVED2[109];
	__IO uint32_t INTENSET; // Offset: 0x304 - Interrupt Enable Set Register
	__IO uint32_t INTENCLR; // Offset: 0x308 - Interrupt Enable Clear Register
	__I uint32_t RESERVED3[13];
	__IO uint32_t EVTEN; // Offset: 0x340 - Event Enable/Disable Register
	__IO uint32_t EVTENSET; // Offset: 0x344 - Event Enable Set Register
	__IO uint32_t EVTENCLR; // Offset: 0x348 - Event Enable Clear Register
	__I uint32_t RESERVED4[110];
	__I uint32_t COUNTER; // Offset: 0x504 - Current COUNTER Value
	__IO uint32_t PRESCALER; // Offset: 0x508 - 12-bit Prescaler for COUNTER Freq. (32768 / (PRESCALER + 1)).
	__I uint32_t RESERVED5[13];
	__IO uint32_t CC[4]; // Offset: 0x540 - Compare Registers
} RTC_t;

/* INTENSET - Interrupt Enable Set Register */
#define RTC_INTENSET_COMPARE0_Pos (16UL)
#define RTC_INTENSET_COMPARE0_Msk (0x01U << RTC_INTENSET_COMPARE0_Pos)

/* EVTEN - Event Enable/Disable Register */
#define RTC_EVTEN_COMPARE0_Pos (16UL)
#define RTC_EVTEN_COMPARE0_Msk (0x01U << RTC_EVTEN_COMPARE0_Pos)

#endif
