#ifndef CORE_INSTRUCTIONS_H
#define CORE_INSTRUCTIONS_H

/**
 * \brief Instruction Synchronization Barrier
 * Instruction Synchronization Barrier flushes the pipeline in the process, so that all
 * instructions following the ISB are fetched from cache or memory, after the instruction has been
 * completed.
 */
__attribute__((always_inline)) static inline void __ISB(void) {
	asm volatile("isb 0x0F":::"memory");
}

/**
 * \brief Data Synchronization Barrier
 * This function acts as a special kind of Data Memory Barrier. It completes when all explicit
 * memory accesses before this instruction complete.
 */
__attribute__((always_inline)) static inline void __DSB(void) {
	asm volatile("dsb 0x0F":::"memory");
}

#endif
