#include "nrf52.h"

//initialize segments
extern uint32_t __stack_top;
extern uint32_t __text_start;
extern uint32_t __etext;
extern uint32_t __data_start;
extern uint32_t __data_end;
extern uint32_t __bss_start;
extern uint32_t __bss_end;

int main(void);
void __libc_init_array(void);

// default empty handler
void Dummy_Handler(void);

// Cortex-M4 core handlers
void NMI_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void HardFault_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void MemManage_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void BusFault_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void UsageFault_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void SVC_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void DebugMon_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void PendSV_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void SysTick_Handler(void) __attribute__((weak, alias("Dummy_Handler")));

// Cortex-M4 peripheral handlers
void PowerClk_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void Radio_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void UART0_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void SPI0_TWI0_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void SPI1_TWI1_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void NFC_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void GPIOTE_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void SAADC_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void Timer0_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void Timer1_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void Timer2_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void RTC0_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void Temp_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void RNG_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void ECB_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void CCM_AAR_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void WDT_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void RTC1_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void QDEC_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void COMP_LPCOMP_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void SWI0_EGU0_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void SWI1_EGU1_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void SWI2_EGU2_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void SWI3_EGU3_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void SWI4_EGU4_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void SWI5_EGU5_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void Timer3_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void Timer4_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void PWM0_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void PDM_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void MWU_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void PWM1_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void PWM2_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void SPI2_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void RTC2_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void I2S_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
void FPU_Handler(void) __attribute__((weak, alias("Dummy_Handler")));

// Exception Table
__attribute__((section(".isr_vector")))
const DeviceVectors_t exception_table = {
	/* Configure initial stack pointer using linker-generated symbols. */
	.pvStack = (void*) (&__stack_top),
	.pfnReset_Handler = (void*) Reset_Handler,
	.pfnNMI_Handler = (void*) NMI_Handler,
	.pfnHardFault_Handler = (void*) HardFault_Handler,
	.pfnMemManage_Handler = (void*) MemManage_Handler,
	.pfnBusFault_Handler = (void*) BusFault_Handler,
	.pfnUsageFault_Handler = (void*) UsageFault_Handler,
	.pfnReserved1_Handler = (void*) (0UL), // Reserved
	.pfnReserved2_Handler = (void*) (0UL), // Reserved
	.pfnReserved3_Handler = (void*) (0UL), // Reserved
	.pfnReserved4_Handler = (void*) (0UL), // Reserved
	.pfnSVC_Handler = (void*) SVC_Handler,
	.pfnDebugMon_Handler = (void*) DebugMon_Handler,
	.pfnReserved5_Handler = (void*) (0UL), // Reserved
	.pfnPendSV_Handler = (void*) PendSV_Handler,
	.pfnSysTick_Handler = (void*) SysTick_Handler,

	/* External Interrupts */
	.pfnPowerClk_Handler = (void*) PowerClk_Handler, // 0 - Power/Clock Controller
	.pfnRadio_Handler = (void*) Radio_Handler, // 1 - Radio Controller
	.pfnUART0_Handler = (void*) UART0_Handler, // 2 - UART0
	.pfnSPI0_TWI0_Handler = (void*) SPI0_TWI0_Handler, // 3 - SPI0/TWI0 Controller
	.pfnSPI1_TWI1_Handler = (void*) SPI1_TWI1_Handler, // 4 - SPI1/TWI1 Controller
	.pfnNFC_Handler = (void*) NFC_Handler, // 5 - Near-Field Communication Controller
	.pfnGPIOTE_Handler = (void*) GPIOTE_Handler, // 6 - General Purpose Input Output Task and Events
	.pfnSAADC_Handler = (void*) SAADC_Handler, // 7 - Successive Approximation Analog-to-Digital Converter
	.pfnTimer0_Handler = (void*) Timer0_Handler, // 8 - Timer 0
	.pfnTimer1_Handler = (void*) Timer1_Handler, // 9 - Timer 1
	.pfnTimer2_Handler = (void*) Timer2_Handler, // 10 - Timer 2
	.pfnRTC0_Handler = (void*) RTC0_Handler, // 11 - Real Time Clock 0
	.pfnTemp_Handler = (void*) Temp_Handler, // 12 - Temperature Sensor
	.pfnRNG_Handler = (void*) RNG_Handler, // 13 - Random Number Generator
	.pfnECB_Handler = (void*) ECB_Handler, // 14 - AES Electronic Code Book
	.pfnCCM_AAR_Handler = (void*) CCM_AAR_Handler, // 15 - AES CCM
	.pfnWDT_Handler = (void*) WDT_Handler, // 16 - Watchdog Timer
	.pfnRTC1_Handler = (void*) RTC1_Handler, // 17 - Real Time Clock 1
	.pfnQDEC_Handler = (void*) QDEC_Handler, // 18 - Quadrature Decoder
	.pfnCOMP_LPCOMP_Handler = (void*) COMP_LPCOMP_Handler, // 19 - Comparator/Low Power Comparator
	.pfnSWI0_EGU0_Handler = (void*) SWI0_EGU0_Handler, // 20 - SWI0/EGU0
	.pfnSWI1_EGU1_Handler = (void*) SWI1_EGU1_Handler, // 21 - SWI1/EGU1
	.pfnSWI2_EGU2_Handler = (void*) SWI2_EGU2_Handler, // 22 - SWI2/EGU2
	.pfnSWI3_EGU3_Handler = (void*) SWI3_EGU3_Handler, // 23 - SWI3/EGU3
	.pfnSWI4_EGU4_Handler = (void*) SWI4_EGU4_Handler, // 24 - SWI4/EGU4
	.pfnSWI5_EGU5_Handler = (void*) SWI5_EGU5_Handler, // 25 - SWI5/EGU5
	.pfnTimer3_Handler = (void*) Timer3_Handler, // 26 - Timer 3
	.pfnTimer4_Handler = (void*) Timer4_Handler, // 27 - Timer 4
	.pfnPWM0_Handler = (void*) PWM0_Handler, // 28 - Pulse-Width Modulation Controller 0
	.pfnPDM_Handler = (void*) PDM_Handler, // 29 - Pulse-Density Modulation Controller
	.pfnReserved6_Handler = (void*) (0UL), // 30 - Reserved
	.pfnReserved7_Handler = (void*) (0UL), // 31 - Reserved
	.pfnMWU_Handler = (void*) MWU_Handler, // 32 - Memory Watch Unit
	.pfnPWM1_Handler = (void*) PWM1_Handler, // 33 - Pulse-Width Modulation Controller 1
	.pfnPWM2_Handler = (void*) PWM2_Handler, // 34 - Pulse-Width Modulation Controller 2
	.pfnSPI2_Handler = (void*) SPI2_Handler, // 35 - Serial Peripheral Interface 2
	.pfnRTC2_Handler = (void*) RTC2_Handler, // 36 - Real Time Clock 2
	.pfnI2S_Handler = (void*) I2S_Handler, // 37 - Inter-IC Sound Controller
	.pfnFPU_Handler = (void*) FPU_Handler // 38 - Floating-Point Unit
};

/*
 * \brief This is the function that gets called on processor reset. To initialize the device, and
 * call the main() routine.
 */
void Reset_Handler(void) {
	uint32_t *pSrc, *pDest;

	// initialize the relocate segment
	pSrc = &__etext;
	pDest = &__data_start;

	if (pSrc != pDest) {
		for (; pDest < &__data_end;) {
			*pDest++ = *pSrc++;
		}
	}

	// clear the zero segment
	for (pDest = &__bss_start; pDest < &__bss_end;) {
		*pDest++ = 0;
	}

	// set the vector table base address
	pSrc = (uint32_t*) &__text_start;
	SCB->VTOR = ((uint32_t) pSrc & SCB_VTOR_TBLOFF_Msk);

	// initialize the C library
	__libc_init_array();

	// branch to main function
	main();

	// infinite loop if we return from main
	while (1);
}

/*
 * \brief Default interrupt handler for unused IRQs.
 */
void Dummy_Handler(void) {
	while (1);
}
